#!/bin/sh
##############################################################################
# (C) Copyright 2012-2015 Hewlett Packard Enterprise Development Company, L.P.
#     Confidential commercial computer software.
#
# 	HP Serviceguard Sybase Toolkit Module script      	
##############################################################################
# haase.sh
#
# Version A.12.00.30
##############################################################################
# REVISION HISTORY:
#     	Version        	mm/dd/yyyy      Changes
#       01.00.00        04/10/2013      New Toolkit design on Linux
#       12.00.00        11/29/2013      Added code for SG online modification failure check
#       12.00.10        07/18/2014      Removed redundant call to function validate_ase during
#                                       start up.
#       12.00.30        06/29/2015      Added changes to support ASE 16.0
##############################################################################

# Source cluster environment
if [[ -f /etc/cmcluster.conf ]]; then
        . /etc/cmcluster.conf
fi

if [[ -z "${SG_SYBASE_MOD}" || ! ${SG_SYBASE_MOD} ]] ; then
# Source SG_UTILS & Sybase Toolkit global variables/functions
        . $SGROOT/sybasetoolkit/sybasetkit_utils.sh
fi

######################################################
#
# VALIDATION/VERIFICATION for Supported actions
#
######################################################

#Validate the package attributes for ASE module
function validate_ase
{

err_flag=0

if [[ -z $SG_PRIV_CLUSTER_VERIFICATION  || $SG_COMMAND = "cmapplyconf" ]]; then
# Check if package is applied
    $CMVIEWCL -f line -s config -p $SG_PACKAGE > /dev/null 2>&1
    if [[ $? -eq 0 ]]; then
        # Check if package is running then only call function verify_unsupported_online_modification
        pkg_stat=`$CMVIEWCL -f line | $GREP "$SG_PACKAGE|status" | $CUT -d "=" -f 2`
        if [[ "$pkg_stat" = "up" && "$1" != "start" ]]; then
            if [[ "${PREV_SG_MODULE_NAME[@]}" = *tkit/sybase/ase* ]] ; then
               verify_unsupported_online_modification
               if [[ $? -eq 1 ]]; then
                  exit 1
               fi
            fi
        fi
    fi
fi

# Confirm that the Sybase Admin user exist!
if ! $ID $ASE_ADMIN_NAME >$NULL 2>&1 ; then
     log_message 0 "ERROR: ASE admin user $ASE_ADMIN_NAME does not exist"
     log_message 0 " - Add user $ASE_ADMIN_NAME on the node ${HOST}."
     err_flag=1
fi

ASE_SHARED=false
if [[ -n "${SG_FS_DIRECTORY[@]}" ]] ; then
    for (( i=0; i<${#SG_FS_DIRECTORY[@]}; i++)) ; do
         if $ECHO $ASE_INSTALL_HOME | $GREP -q -w ${SG_FS_DIRECTORY[$i]} 2>$NULL ; then
                ASE_SHARED=true
         fi
    done
fi

if ! ${ASE_SHARED} || [[ "$1" = "start" ]] 
then  
    if [[ ! -d $ASE_INSTALL_HOME ]] ; then
        log_message 0 "ERROR: ase/INSTALL_HOME - $ASE_INSTALL_HOME directory does not exist."
        return 1
    fi

    if [[ ! -d $ASE_ASE_INSTALL_PATH ]] ; then
        log_message 0 "ERROR: ase/ASE_INSTALL_PATH - $ASE_ASE_INSTALL_PATH directory does not exist."
        err_flag=1
    fi

    if [[ ! -d $ASE_OCS_INSTALL_PATH ]] ; then
        log_message 0 "ERROR: ase/OCS_INSTALL_PATH - $ASE_OCS_INSTALL_PATH directory does not exist."
        err_flag=1
    fi

    if [[ ! -f $ASE_RUNSCRIPT ]] ; then
        log_message 0 "ERROR: ase/RUNSCRIPT - $ASE_RUNSCRIPT does not exist, provide run script's path and name appropriately."
        err_flag=1
    fi

    if [[ ! -f ${ASE_INSTALL_HOME}/SYBASE.sh ]]; then
        log_message 0 "ERROR: SYBASE.sh file is missing in directory ${ASE_INSTALL_HOME}."
        err_flag=1
    fi

    if [[ ! -f $ASE_PASSWD_FILE ]] ; then
        log_message 0 "ERROR: ase/PASSWD_FILE - $ASE_PASSWD_FILE does not exist."
        log_message 0 " - Create the password file which is required for health check and graceful shutdown."
        err_flag=1
    fi
   
    if [[ ! -x ${ASE_OCS_INSTALL_PATH}/bin/isql ]]; then
        log_message 0 "ERROR: Executable - \"$ASE_OCS_INSTALL_PATH/bin/isql\" does not exist"
        log_message 0 " - Make sure that isql is present in directory - $ASE_OCS_INSTALL_PATH/bin"
        err_flag=1
    fi
fi
#end of Non-Shared/Startup check

if [[ $err_flag -eq 1 ]]; then
    log_message 5 "ASE Module verification failed..."
    return 1
fi

log_message 5 "ASE Module verification success..."
return 0
}

###################################################
#
# START SYBASE ASE SERVER
#
###################################################

function start_ase
{
    log_message ${ASE_RUNSCRIPT}
    if [[ ! -f ${ASE_RUNSCRIPT} ]]
    then
        log_message 0 "ASE: ERROR: ASE RUN Server file '${ASE_RUNSCRIPT}' does not exist."
        log_message 0 "ASE: ERROR: Cannot start '${ASE_INSTANCE_NAME}'."
        check_return 1 1
    else
        #
        # Proceed to boot up ASE
        #

        if [[ -f $ASE_INSTALL_HOME/SYBASE.sh ]] ; then
            . $ASE_INSTALL_HOME/SYBASE.sh
            export PATH=$PATH:${ASE_ASE_INSTALL_PATH}/bin
        else
            $ECHO "ERROR: SYBASE.sh file is missing in directory ${ASE_INSTALL_HOME}"
            exit 1
        fi
        $SU - ${ASE_ADMIN_NAME} -c "/bin/sh" << EOF
        . $ASE_INSTALL_HOME/SYBASE.sh
        startserver -f ${ASE_RUNSCRIPT} >> ${ASE_CONSOLE_LOGFILE} 2>&1
EOF
        typeset -i chktime=0
        typeset -i counter=10
        while [[ $chktime -le ${ASE_RECOVERY_TIMEOUT} ]]
        do
            ${ASE_ASE_INSTALL_PATH}/install/showserver|$GREP -i "dataserver" | $GREP -i "\-s${ASE_INSTANCE_NAME}" |$GREP  "\-d"  2>&1
            if [[ $? -ne 0 ]]
            then
                 chktime=`expr $chktime + $counter`
            else
                log_message 0 "ASE: ASE server '${ASE_INSTANCE_NAME}' started successfully."
                break
            fi
                $SLEEP $counter
        done

        if [[ $chktime -gt ${ASE_RECOVERY_TIMEOUT} ]]
        then
            check_return 1 1
        fi
    fi
}

######################################################
#
# STOP SYBASE ASE SERVER
#
######################################################

function stop_ase
{
log_message 0 "ASE: Stopping Sybase ASE session..."
# If the server is up and running, then we shut it down.

if [[ -f ${ASE_PASSWD_FILE} ]] ; then
    PASSWD=`$CAT ${ASE_PASSWD_FILE} | $HEAD -n 1 | $AWK '{print $1}'` 2>&1
    if [[ -z "${PASSWD}" ]]; then
        log_message 0 "ASE: No password in file ${ASE_PASSWD_FILE}."
    else
        if [[ -f $ASE_INSTALL_HOME/SYBASE.sh ]] ; then
            . $ASE_INSTALL_HOME/SYBASE.sh
            export PATH=$PATH:${ASE_ASE_INSTALL_PATH}/bin
        else
            $ECHO "ERROR: SYBASE.sh file is missing in directory ${ASE_INSTALL_HOME}"
            exit 1
        fi
        $SU - ${ASE_ADMIN_NAME} -c "/bin/sh" << EOF >/dev/null 2>&1
        . ${ASE_INSTALL_HOME}/SYBASE.sh
        ${ASE_OCS_INSTALL_PATH}/bin/isql -U${ASE_USER_NAME} -P${PASSWD} -S${ASE_INSTANCE_NAME} >/dev/null 2>&1 
shutdown
go
quit
EOF
        $SLEEP ${ASE_TIME_OUT}
        if ! $PS -ef | $GREP -i "dataserver" | $GREP -e -s${ASE_INSTANCE_NAME} | grep -v grep > /dev/null 2>&1 ; then
            log_message 0 "ASE: ASE - ${ASE_INSTANCE_NAME} is stopped successfully"
            unset MONITOR_PROCESSES[@]
        else
            log_message 0 "ASE: Could not stop Sybase ASE instance - ${ASE_INSTANCE_NAME}. Aborting Sybase."
        fi
    fi
else
    log_message 0 "ASE: Password file ${ASE_PASSWD_FILE} does not exist"
fi

# Forceful shutdown
c=0
typeset -i num_procs=${#MONITOR_PROCESSES[@]}
while [[ $c -lt $num_procs ]]
do
        # The shutdown was not successful; therefore, get rid of any existing
        # hung processes by kill.
   pid=`$PS -ef | $GREP -i "${MONITOR_PROCESSES[$c]}" | $GREP -e "-s${ASE_INSTANCE_NAME}" | $AWK '{print $2}'`
   if [[ -n $pid ]]
   then
        $KILL -9 $pid > /dev/null 2>&1
        log_message 0 "ASE: Success in killing ${MONITOR_PROCESSES[$c]}"
        unset ${MONITOR_PROCESSES[$c]}
   fi
c=`expr $c + 1`
done
return 0
}

###############################################################
#
# VERIFY UNSUPPORTED ONLINE MODIFICATION FUNCTION
#
###############################################################

function verify_unsupported_online_modification
{

typeset -i i=0
typeset -i flag=0
# These arrays define following things
# curarrayname => Attribute Names for which online modifcation is not supported.
# cuarray => Current values of the unsupported attribitues
# prevarray => Previous values of unsupported attribtes.

declare -a curarrayname=( "ase/INSTALL_HOME" "ase/ASE_INSTALL_PATH" "ase/OCS_INSTALL_PATH" "ase/INSTANCE_NAME" "ase/ADMIN_NAME" )
declare -a curarray=( "$ASE_INSTALL_HOME" "$ASE_ASE_INSTALL_PATH" "$ASE_OCS_INSTALL_PATH" "$ASE_INSTANCE_NAME" "$ASE_ADMIN_NAME" )
declare -a prevarray=( "$PREV_ASE_INSTALL_HOME" "$PREV_ASE_ASE_INSTALL_PATH" "$PREV_ASE_OCS_INSTALL_PATH" "$PREV_ASE_INSTANCE_NAME" "$PREV_ASE_ADMIN_NAME" )
while (( $i <= ${#curarray[@]} -1 && $i <= ${#prevarray[@]} -1 && $i <= ${#curarrayname[@]} -1 ))
do
  if [[ "${curarray[i]}" != "${prevarray[i]}" ]]; then
      sg_log 0 "ERROR: Online modification of the attribute ${curarrayname[i]} is not supported"
      flag=1
  fi
  (( i = $i + 1 ))
done
  if [[ $flag -eq 1 ]]; then
      return 1
  else 
      return 0  
  fi

}

###############################################################
#
# RECONFIGURE STOP FUNCTION
#
###############################################################

function reconfigure_stop_function
{
  if ! sg_check_if_reconfig_has_failed ; then 
     validate_reconfigure_restart
     restart_check=$?
     if [[ $restart_check -eq 1 ]] ; then
        if ! ${SGSBIN}/cmhaltserv ${service_name} ; then
           sg_log 999 "ERROR: Could not stop service: ${service_name}"
           sg_set_reconfig_failure
           exit_val=1 
        else
           sg_log 999 "SUCCESS: Successfully stopped service: $service_name"
           sg_log 0 "SUCCESS: Successfully stopped service: $service_name"
        fi
     fi
  else
     sg_log 999 "WARNING: Not attempting to stop the service: ${service_name}"
  fi
}

###############################################################
#
# RECONFIGURE START FUNCTION
#
###############################################################

function reconfigure_start_function
{
    validate_reconfigure_restart
    if [[ $? -eq 1 ]]; then
         msg="cmrunserv: Unable to start service $service_name (it is already active)"
         out=`${SGSBIN}/cmrunserv ${service_name} "${service_cmd_string}" 2>&1`
         status=$? 
         if [[ $status -ne 0 ]] && [[ "$msg" != "$out" ]] ; then
            sg_log 999 "ERROR: Failed to start the service $service_name after online package modification"
            sg_set_reconfig_failure
            exit_val=1
         elif [[ $status -eq 0 ]] ; then
            sg_log 999 "SUCCESS: Successfully started service: $service_name"
            if ! sg_check_if_reconfig_has_failed ; then
               log_message 0 "SUCCESS: Successfully started service: $service_name"
            fi
         fi
    fi
}

################################################################
#
# VALIDATE RECONFIGURE RESTART FUNCTION
#
################################################################

function validate_reconfigure_restart
{
#Check for online modification was done to email attribute.
sup_attri_changed=0

var1=`$ECHO ${SG_EMAIL_ID[*]} | $SED 's/ //g'`
var2=`$ECHO ${PREV_SG_EMAIL_ID[*]} | $SED 's/ //g'`
if [[ "$var1" != "$var2" ]]; then
    sup_attri_changed=1
fi

#Check for online modification done to MONITOR_PROCESSES attribute
var3=`$ECHO ${ASE_MONITOR_PROCESSES[*]} | $SED 's/ //g'`
var4=`$ECHO ${PREV_ASE_MONITOR_PROCESSES[*]} | $SED 's/ //g'`
if [[ "$var3" != "$var4" ]]; then
    sup_attri_changed=1
fi

#Checking if other supported attributes are changed or not
if [[ "$ASE_MONITOR_INTERVAL" != "$PREV_ASE_MONITOR_INTERVAL" ]] || 
[[ "$ASE_USER_NAME" != "$PREV_ASE_USER_NAME" ]] || 
[[ "$ASE_PASSWD_FILE" != "$PREV_ASE_PASSWD_FILE" ]] || 
[[ "$ASE_TIME_OUT" != "$PREV_$ASE_TIME_OUT" ]] ||
[[ "$ASE_RETRY_TIME" != "$PREV_ASE_RETRY_TIME" ]] ||  
[[ "$SG_SCRIPT_LOG_FILE" != "$PREV_SG_SCRIPT_LOG_FILE" ]]; then
    sup_attri_changed=1
fi

# Based on whether supported attribute changed or not and unsupported attribute
# changed or not, take decision of restarting the service.
# if sup_attri_changed=1 then service will be restarted
# if sup_attri_changed=0 then service will not be restarted

return $sup_attri_changed

}

######################### MAIN ####################################

Usage_Msg="Usage: ${0} [ start | stop | validate | ase_monitor | reconfigure_stop | reconfigure_start ]"
if [[ $1 = "" ]]
then
    log_message ${Usage_Msg}
    exit 0
fi

log_message 5 "${CMD} called with argument(s): $*"

typeset -i exit_val=0
#Identify the service name and service command for the package 
if [[ $1 = "reconfigure_stop" || $1 = "reconfigure_start" ]]; then
    if [[ ${#SG_SERVICE_CMD[@]} -ne 0 ]]; then
        i=0
        service_name=""
        service_cmd_string=""
        for cmd in "${SG_SERVICE_CMD[@]}"
        do
          $ECHO "$cmd" | $GREP "ase_monitor" >> /dev/null
          if [[ $? -eq 0 ]]
          then
              service_name=${SG_SERVICE_NAME[$i]}
              service_cmd_string="$cmd"
              break
          fi
          i=$i+1
        done
     fi
fi

declare -a MONITOR_PROCESSES
MONITOR_PROCESSES=(${ASE_MONITOR_PROCESSES[@]})

case $1 in
   start)
             if sg_check_if_reconfig_has_failed ; then
                log_message 999 "WARNING: Not attempting to start Sybase ASE module"
                return
             else
       	        if ! start_ase ; then
                   if sg_check_if_it_is_reconfiguration ; then
                      log_message 999 "ERROR: Failed to start Sybase ASE module"
                      sg_set_reconfig_failure
                   fi
                   check_return 1 1
                else
                   if sg_check_if_it_is_reconfiguration ; then
                      log_message 999 "SUCCESS: Successfully started Sybase ASE module"
                   fi
                fi
             fi
       ;;
   stop)
                stop_ase
                exit_val=$?
       ;;
   ase_monitor)  
                . $SGROOT/sybasetoolkit/haase.mon
                mon_ret=$?
                if [[ $mon_ret -ne 0 ]]; then
                    check_return 1 3
                else
                    exit_val=mon_ret
                fi
       ;;
   validate)
		validate_ase
		exit_val=$? 
       ;;
   reconfigure_stop)
                
                reconfigure_stop_function
       ;;
   reconfigure_start)

                reconfigure_start_function
       ;;
    *)

                sg_log 0 "${Usage_Msg}"
        	exit_val=1
       ;;
esac

log_message 5 "ASE: End of ${CMD} module"

exit ${exit_val}

