#!/bin/sh
##############################################################################
# (C) Copyright 2012-15 Hewlett-Packard Development Company, L.P.
#     Confidential commercial computer software.
#
# 	     	
##############################################################################
# sybasetkit_utils.sh :	HP Serviceguard toolkit for Sybase - common module 
#
# Version A.12.00.30
##############################################################################
# REVISION HISTORY:
#       Version	        mm/dd/yyyy      Changes
#       01.00.00        04/10/2013      Sybase Toolkit on linux new release
#       12.00.30        06/29/2015      Removed references to /etc/SuSE-release file.
#
##############################################################################
#initialize module to false and set true once at the end of the module
SG_SYBASE_MOD=false

############################################
# Global Aliases/Variables
############################################

#Common executables path
SU=/bin/su
RM=/bin/rm
TR=/bin/tr
ECHO=/bin/echo
AWK=/bin/awk
SORT=/bin/sort
GREP=/bin/grep
PS=/bin/ps
CAT=/bin/cat
KILL=/bin/kill
SED=/bin/sed
SLEEP=/bin/sleep
ID=/usr/bin/id
HEAD=/usr/bin/head
NULL=/dev/null

HOSTNAME=/bin/hostname
CMVIEWCL=$SGSBIN/cmviewcl

typeset TKIT_SCRIPT_DIR=$SGROOT/sybasetoolkit
typeset -i exit_val=0


ASETKIT=false
ASESAP=false
REPSERV=false
DRAGENT=false


#############################################
#Linux Release & version checks
#############################################

#check_distro_type

if [[ -f /etc/redhat-release ]] ; then
        #write all RHEL related pre-requisite assignments and checks
        CUT=/bin/cut 
else
        #write all SUSE related pre-requisite assignments and checks
        CUT=/usr/bin/cut
fi

HOST=`$HOSTNAME | $CUT -d "." -f 1`

##############################
# Source SG utility functions.
##############################


if [[ -z "$SG_UTILS" ]]; then
    SG_UTILS=$SGCONF/scripts/mscripts/utils.sh
fi

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        $ECHO "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    $ECHO "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
# Get the environment for this package through utility function
# sg_source_pkg_env().
###################################################################

export PATH=$PATH:$SGSBIN

if [[ ! -f $2 ]] ; then
        sg_source_pkg_env $1
else
        sg_source_pkg_env $*
fi

#Dont traverse modules if already done
if [[ -z "${MODULES_PROCESSED}" || ! ${MODULES_PROCESSED} ]] ; then

#modules definitions for validations
typeset eSAP_MODULE=sgesap/dbinstance
typeset DB_MODULE=tkit/sybase/ase
typeset RS_MODULE=tkit/sybase/rs
typeset DRA_MODULE=tkit/sybase/dra
typeset EMAIL_MODULE=sg/email

for ((i=0 ; i<${#SG_MODULE_NAME[*]} ; i++ ))
do
  if [[ "${SG_MODULE_NAME[$i]}" == "${eSAP_MODULE}" ]] ; then
        ASESAP=true
  elif [[ "${SG_MODULE_NAME[$i]}" == "${DB_MODULE}" ]] ; then
        ASETKIT=true
  elif [[ "${SG_MODULE_NAME[$i]}" == "${RS_MODULE}" ]] ; then
        REPSERV=true
  elif [[ "${SG_MODULE_NAME[$i]}" == "${DRA_MODULE}" ]] ; then
        DRAGENT=true
  fi
done

MODULES_PROCESSED=true

fi

############################
# Function to log message 
############################

function log_message
{
    if [[ $# -eq 1 ]]; then
        echo "$(date '+%b %e %T') - \"${HOST}\": $1 "
    else 
	sg_log $1 "${2}"
    fi
}


############################-----------------###################################
# Function: check_return
# Check return value of functions and exit. It accepts two parameters.
# Parameters:
# 1. Exit value of the previously executed command
# 2. The place of execution
#        1 - start server
#        2 - stop server
#        3 - Monitor/Health check 
#        4 - Improper parameters 
############################-----------------###################################

function check_return
{
    if (( $1 != 0 ))
    then
    case $2 in
        1)
         log_message 0 "ERROR: Failed to start server - check the appropriate logfile for possible error."
         exit 1
         ;;

        2)
         log_message 0 "ERROR: Failed to stop server - check the appropriate logfile for possible error."
         exit 1
         ;;

        3)
         log_message 0 "ERROR: Monitor Function failed"
         exit 1
         ;;

        4)
         log_message 0 "ERROR: Improper attribute configuration for the package ${SG_PACKAGE}."
         exit 1
         ;;

        *)
         log_message 0 "ERROR: Failed, unknown error."
	 exit 1
         ;;
    esac
    fi
}


#############################-----------------###################################
# Function: to_lowercase
#
# This function will convert the string passed to lowercase and prints it.
# Arguements:
# arg1 - String to be converted
#############################-----------------###################################

function to_lowercase
{
        /bin/echo $1| $TR "[:upper:]" "[:lower:]" | $TR -d "[:space:]"
}


SG_SYBASE_MOD=true
